@@ -2,24 +2,17 @@
 # 2.0, and the BSD License. See the LICENSE file in the root of this repository
 # for complete details.
 
+from __future__ import annotations
 
 INCLUDES = """
 #include <openssl/err.h>
 """
 
 TYPES = """
-static const int CIPHER_R_DATA_NOT_MULTIPLE_OF_BLOCK_LENGTH;
-
 static const int EVP_F_EVP_ENCRYPTFINAL_EX;
 static const int EVP_R_DATA_NOT_MULTIPLE_OF_BLOCK_LENGTH;
-static const int EVP_R_BAD_DECRYPT;
-static const int EVP_R_UNSUPPORTED_PRIVATE_KEY_ALGORITHM;
-static const int PKCS12_R_PKCS12_CIPHERFINAL_ERROR;
-static const int EVP_R_XTS_DUPLICATED_KEYS;
 
 static const int ERR_LIB_EVP;
-static const int ERR_LIB_PROV;
-static const int ERR_LIB_PKCS12;
 
 static const int SSL_TLSEXT_ERR_OK;
 static const int SSL_TLSEXT_ERR_ALERT_FATAL;
@@ -43,25 +36,9 @@ int ERR_GET_REASON(unsigned long);
 """
 
 CUSTOMIZATIONS = """
-/* This define is tied to provider support and is conditionally
-   removed if Cryptography_HAS_PROVIDERS is false */
-#ifndef ERR_LIB_PROV
-#define ERR_LIB_PROV 0
-#endif
-
-#if !CRYPTOGRAPHY_OPENSSL_111D_OR_GREATER || CRYPTOGRAPHY_IS_BORINGSSL
-static const int EVP_R_XTS_DUPLICATED_KEYS = 0;
-#endif
-
-#if CRYPTOGRAPHY_IS_BORINGSSL
-static const int ERR_LIB_PKCS12 = 0;
+#if CRYPTOGRAPHY_IS_BORINGSSL || CRYPTOGRAPHY_IS_AWSLC
 static const int EVP_F_EVP_ENCRYPTFINAL_EX = 0;
-static const int EVP_R_BAD_DECRYPT = 0;
 static const int EVP_R_DATA_NOT_MULTIPLE_OF_BLOCK_LENGTH = 0;
-static const int EVP_R_UNSUPPORTED_PRIVATE_KEY_ALGORITHM = 0;
-static const int PKCS12_R_PKCS12_CIPHERFINAL_ERROR = 0;
-#else
-static const int CIPHER_R_DATA_NOT_MULTIPLE_OF_BLOCK_LENGTH = 0;
 #endif
 
 /* SSL_R_UNEXPECTED_EOF_WHILE_READING is needed for pyOpenSSL
