@@ -2,30 +2,32 @@
 // 2.0, and the BSD License. See the LICENSE file in the root of this repository
 // for complete details.
 
+use cryptography_x509::ocsp_req::{self, OCSPRequest as RawOCSPRequest};
+use cryptography_x509::{common, oid};
+use pyo3::types::{PyAnyMethods, PyListMethods};
+
 use crate::asn1::{big_byte_slice_to_py_int, oid_to_py_oid, py_uint_to_big_endian_bytes};
 use crate::error::{CryptographyError, CryptographyResult};
 use crate::x509::{extensions, ocsp};
-use crate::{exceptions, x509};
-use cryptography_x509::{common, ocsp_req, oid};
-use pyo3::IntoPy;
+use crate::{exceptions, types, x509};
 
-#[ouroboros::self_referencing]
-struct OwnedOCSPRequest {
-    data: pyo3::Py<pyo3::types::PyBytes>,
-    #[borrows(data)]
-    #[covariant]
-    value: ocsp_req::OCSPRequest<'this>,
-}
+self_cell::self_cell!(
+    struct OwnedOCSPRequest {
+        owner: pyo3::Py<pyo3::types::PyBytes>,
+        #[covariant]
+        dependent: RawOCSPRequest,
+    }
+);
 
-#[pyo3::prelude::pyfunction]
-fn load_der_ocsp_request(
+#[pyo3::pyfunction]
+pub(crate) fn load_der_ocsp_request(
     py: pyo3::Python<'_>,
     data: pyo3::Py<pyo3::types::PyBytes>,
 ) -> CryptographyResult<OCSPRequest> {
     let raw = OwnedOCSPRequest::try_new(data, |data| asn1::parse_single(data.as_bytes(py)))?;
 
     if raw
-        .borrow_value()
+        .borrow_dependent()
         .tbs_request
         .request_list
         .unwrap_read()
@@ -41,21 +43,21 @@ fn load_der_ocsp_request(
 
     Ok(OCSPRequest {
         raw,
-        cached_extensions: None,
+        cached_extensions: pyo3::sync::GILOnceCell::new(),
     })
 }
 
-#[pyo3::prelude::pyclass(module = "cryptography.hazmat.bindings._rust.ocsp")]
-struct OCSPRequest {
+#[pyo3::pyclass(frozen, module = "cryptography.hazmat.bindings._rust.ocsp")]
+pub(crate) struct OCSPRequest {
     raw: OwnedOCSPRequest,
 
-    cached_extensions: Option<pyo3::PyObject>,
+    cached_extensions: pyo3::sync::GILOnceCell<pyo3::PyObject>,
 }
 
 impl OCSPRequest {
     fn cert_id(&self) -> ocsp_req::CertID<'_> {
         self.raw
-            .borrow_value()
+            .borrow_dependent()
             .tbs_request
             .request_list
             .unwrap_read()
@@ -66,7 +68,7 @@ impl OCSPRequest {
     }
 }
 
-#[pyo3::prelude::pymethods]
+#[pyo3::pymethods]
 impl OCSPRequest {
     #[getter]
     fn issuer_name_hash(&self) -> &[u8] {
@@ -82,12 +84,11 @@ impl OCSPRequest {
     fn hash_algorithm<'p>(
         &self,
         py: pyo3::Python<'p>,
-    ) -> Result<&'p pyo3::PyAny, CryptographyError> {
+    ) -> Result<pyo3::Bound<'p, pyo3::PyAny>, CryptographyError> {
         let cert_id = self.cert_id();
 
-        let hashes = py.import(pyo3::intern!(py, "cryptography.hazmat.primitives.hashes"))?;
         match ocsp::ALGORITHM_PARAMETERS_TO_HASH.get(&cert_id.hash_algorithm.params) {
-            Some(alg_name) => Ok(hashes.getattr(*alg_name)?.call0()?),
+            Some(alg_name) => Ok(types::HASHES_MODULE.get(py)?.getattr(*alg_name)?.call0()?),
             None => Err(CryptographyError::from(
                 exceptions::UnsupportedAlgorithm::new_err(format!(
                     "Signature algorithm OID: {} not recognized",
@@ -101,22 +102,21 @@ impl OCSPRequest {
     fn serial_number<'p>(
         &self,
         py: pyo3::Python<'p>,
-    ) -> Result<&'p pyo3::PyAny, CryptographyError> {
+    ) -> Result<pyo3::Bound<'p, pyo3::PyAny>, CryptographyError> {
         let bytes = self.cert_id().serial_number.as_bytes();
         Ok(big_byte_slice_to_py_int(py, bytes)?)
     }
 
     #[getter]
-    fn extensions(&mut self, py: pyo3::Python<'_>) -> pyo3::PyResult<pyo3::PyObject> {
-        let tbs_request = &self.raw.borrow_value().tbs_request;
+    fn extensions(&self, py: pyo3::Python<'_>) -> pyo3::PyResult<pyo3::PyObject> {
+        let tbs_request = &self.raw.borrow_dependent().tbs_request;
 
-        let x509_module = py.import(pyo3::intern!(py, "cryptography.x509"))?;
         x509::parse_and_cache_extensions(
             py,
-            &mut self.cached_extensions,
+            &self.cached_extensions,
             &tbs_request.raw_request_extensions,
-            |oid, value| {
-                match *oid {
+            |ext| {
+                match ext.extn_id {
                     oid::NONCE_OID => {
                         // This is a disaster. RFC 2560 says that the contents of the nonce is
                         // just the raw extension value. This is nonsense, since they're always
@@ -124,24 +124,21 @@ impl OCSPRequest {
                         // nonce is an OCTET STRING, and so you should unwrap the TLV to get
                         // the nonce. So we try parsing as a TLV and fall back to just using
                         // the raw value.
-                        let nonce = asn1::parse_single::<&[u8]>(value).unwrap_or(value);
-                        Ok(Some(
-                            x509_module.call_method1(pyo3::intern!(py, "OCSPNonce"), (nonce,))?,
-                        ))
+                        let nonce = ext.value::<&[u8]>().unwrap_or(ext.extn_value);
+                        Ok(Some(types::OCSP_NONCE.get(py)?.call1((nonce,))?))
                     }
                     oid::ACCEPTABLE_RESPONSES_OID => {
-                        let oids = asn1::parse_single::<
-                            asn1::SequenceOf<'_, asn1::ObjectIdentifier>,
-                        >(value)?;
+                        let oids = ext.value::<asn1::SequenceOf<'_, asn1::ObjectIdentifier>>()?;
                         let py_oids = pyo3::types::PyList::empty(py);
                         for oid in oids {
                             py_oids.append(oid_to_py_oid(py, &oid)?)?;
                         }
 
-                        Ok(Some(x509_module.call_method1(
-                            pyo3::intern!(py, "OCSPAcceptableResponses"),
-                            (py_oids,),
-                        )?))
+                        Ok(Some(
+                            types::OCSP_ACCEPTABLE_RESPONSES
+                                .get(py)?
+                                .call1((py_oids,))?,
+                        ))
                     }
                     _ => Ok(None),
                 }
@@ -152,63 +149,52 @@ impl OCSPRequest {
     fn public_bytes<'p>(
         &self,
         py: pyo3::Python<'p>,
-        encoding: &pyo3::PyAny,
-    ) -> CryptographyResult<&'p pyo3::types::PyBytes> {
-        let der = py
-            .import(pyo3::intern!(
-                py,
-                "cryptography.hazmat.primitives.serialization"
-            ))?
-            .getattr(pyo3::intern!(py, "Encoding"))?
-            .getattr(pyo3::intern!(py, "DER"))?;
-        if !encoding.is(der) {
+        encoding: &pyo3::Bound<'p, pyo3::PyAny>,
+    ) -> CryptographyResult<pyo3::Bound<'p, pyo3::types::PyBytes>> {
+        if !encoding.is(&types::ENCODING_DER.get(py)?) {
             return Err(pyo3::exceptions::PyValueError::new_err(
                 "The only allowed encoding value is Encoding.DER",
             )
             .into());
         }
-        let result = asn1::write_single(self.raw.borrow_value())?;
+        let result = asn1::write_single(self.raw.borrow_dependent())?;
         Ok(pyo3::types::PyBytes::new(py, &result))
     }
 }
 
-#[pyo3::prelude::pyfunction]
-fn create_ocsp_request(
+#[pyo3::pyfunction]
+pub(crate) fn create_ocsp_request(
     py: pyo3::Python<'_>,
-    builder: &pyo3::PyAny,
+    builder: &pyo3::Bound<'_, pyo3::PyAny>,
 ) -> CryptographyResult<OCSPRequest> {
     let builder_request = builder.getattr(pyo3::intern!(py, "_request"))?;
+    let serial_number_bytes;
+
+    let ka_vec = cryptography_keepalive::KeepAlive::new();
+    let ka_bytes = cryptography_keepalive::KeepAlive::new();
 
     // Declare outside the if-block so the lifetimes are right.
-    let (py_cert, py_issuer, py_hash): (
+    let (py_cert, py_issuer, py_hash, issuer_name_hash, issuer_key_hash): (
         pyo3::PyRef<'_, x509::certificate::Certificate>,
         pyo3::PyRef<'_, x509::certificate::Certificate>,
-        &pyo3::PyAny,
+        pyo3::Bound<'_, pyo3::PyAny>,
+        pyo3::pybacked::PyBackedBytes,
+        pyo3::pybacked::PyBackedBytes,
     );
     let req_cert = if !builder_request.is_none() {
-        let tuple = builder_request.extract::<(
-            pyo3::PyRef<'_, x509::certificate::Certificate>,
-            pyo3::PyRef<'_, x509::certificate::Certificate>,
-            &pyo3::PyAny,
-        )>()?;
-        py_cert = tuple.0;
-        py_issuer = tuple.1;
-        py_hash = tuple.2;
-        ocsp::certid_new(py, &py_cert, &py_issuer, py_hash)?
+        (py_cert, py_issuer, py_hash) = builder_request.extract()?;
+        ocsp::certid_new(py, &ka_bytes, &py_cert, &py_issuer, &py_hash)?
     } else {
-        let (issuer_name_hash, issuer_key_hash, py_serial, py_hash): (
-            &[u8],
-            &[u8],
-            &pyo3::types::PyLong,
-            &pyo3::PyAny,
-        ) = builder
+        let py_serial: pyo3::Bound<'_, pyo3::types::PyInt>;
+        (issuer_name_hash, issuer_key_hash, py_serial, py_hash) = builder
             .getattr(pyo3::intern!(py, "_request_hash"))?
             .extract()?;
-        let serial_number = asn1::BigInt::new(py_uint_to_big_endian_bytes(py, py_serial)?).unwrap();
+        serial_number_bytes = py_uint_to_big_endian_bytes(py, py_serial)?;
+        let serial_number = asn1::BigInt::new(&serial_number_bytes).unwrap();
         ocsp::certid_new_from_hash(
             py,
-            issuer_name_hash,
-            issuer_key_hash,
+            &issuer_name_hash,
+            &issuer_key_hash,
             serial_number,
             py_hash,
         )?
@@ -216,7 +202,9 @@ fn create_ocsp_request(
 
     let extensions = x509::common::encode_extensions(
         py,
-        builder.getattr(pyo3::intern!(py, "_extensions"))?,
+        &ka_vec,
+        &ka_bytes,
+        &builder.getattr(pyo3::intern!(py, "_extensions"))?,
         extensions::encode_extension,
     )?;
     let reqs = [ocsp_req::Request {
@@ -235,12 +223,5 @@ fn create_ocsp_request(
         optional_signature: None,
     };
     let data = asn1::write_single(&ocsp_req)?;
-    load_der_ocsp_request(py, pyo3::types::PyBytes::new(py, &data).into_py(py))
-}
-
-pub(crate) fn add_to_module(module: &pyo3::prelude::PyModule) -> pyo3::PyResult<()> {
-    module.add_function(pyo3::wrap_pyfunction!(load_der_ocsp_request, module)?)?;
-    module.add_function(pyo3::wrap_pyfunction!(create_ocsp_request, module)?)?;
-
-    Ok(())
+    load_der_ocsp_request(py, pyo3::types::PyBytes::new(py, &data).unbind())
 }
